package cz.muni.fi.pb162.radio;

import cz.muni.fi.pb162.radio.parts.*;
/**
 * Universal radio that is possible to build from any parts, provided that they fit the requirements
 * 
 * @author �ubo� Pecho
 * @version 20.10.2009
 */
public class UniversalRadio implements Radio
{

    /**
     * FM frequency, on which the radio should receive signal
     */
    private double frequency = 0;
    
    /**
     * Switch, with which the radio can be turn on and off
     */
    private Switchable radioSwitch;
    
    /**
     * Speaker of the radio
     */
    private GivingSound speaker;
    
    /**
     * Display of the radio
     */
    private Displayable display;

    /**
     * Constructs new instance of Universal radio made off given parts
     * @param radioSwitch
     * @param speaker
     * @param display
     */
    public UniversalRadio(Switchable radioSwitch, GivingSound speaker, Displayable display) {
        this.radioSwitch = radioSwitch;
        this.speaker = speaker;
        this.display = display;
    }
    

    /**
     * Turns the radio on
     */
    public void turnOn() {
        radioSwitch.setTurnedOn(true);
    }

    /**
     * Turns the radio off
     */
    public void turnOff() {
        radioSwitch.setTurnedOn(false);
        display.setContent("");
        display.setContent("");
    }

    /**
     * Increases the volume of the radio by one point
     */
    public void increaseVolume() {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        speaker.setVolume(speaker.getVolume() + 1);
    }

    /**
     * Decreases the volume of the radio by one point
     */
    public void decreaseVolume() {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        speaker.setVolume(speaker.getVolume() - 1);
    }

    /**
     * Sets the frequency, on which the radio should receive signal.
     * Negative frequencies are ignored
     * @param frequency
     */
    public void setFrequency(double frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0) {
            throw new IllegalArgumentException("the frequency must be greater than 0");
        }
        this.frequency = frequency;
        printFrequencyToDisplay();
    }

    /**
     * Increases the frequency by the given value
     * @param frequency the value, by which should be the frequency increased
     */
    public void increaseFrequency(double frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0) {
            throw new IllegalArgumentException("the frequency must be greater than 0");
        }
        this.frequency += frequency;
        printFrequencyToDisplay();
    }

    /**
     * Increases the frequency by the given value
     * @param frequency the value, by which should be the frequency increased
     */
    public void increaseFrequency(int frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0) {
            throw new IllegalArgumentException("the frequency must be greater than 0");
        }
        this.frequency += frequency;
        printFrequencyToDisplay();
    }

    /**
     * Decreases the frequency by the given value
     * @param frequency the value, by which should be the frequency decreased
     */
    public void decreaseFrequency(double frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0 || frequency > this.frequency) {
            throw new IllegalArgumentException("the frequency must be greater than 0" +
                    " and lower or equal than actual set frequency");
        }
        this.frequency -= frequency;
        printFrequencyToDisplay();
    }

    /**
     * Decreases the frequency by the given value
     * @param frequency the value, by which should be the frequency decreased
     */
    public void decreaseFrequency(int frequency) {
        if (!radioSwitch.isTurnedOn()) {
            throw new IllegalStateException("The radio is no turned on");
        }
        if (frequency < 0 || frequency > this.frequency) {
            throw new IllegalArgumentException("the frequency must be greater than 0" +
                    " and lower or equal than actual set frequency");
        }
            this.frequency -= frequency;
            printFrequencyToDisplay();
    }

    /**
     * prints the frequency to the display
     */
    private void printFrequencyToDisplay() {
        display.setContent(((Double) frequency).toString());
    }

    public GivingSound getSpeaker() {
        return speaker;
    }

    @Override
    public boolean equals(Object o) {
        if (this == o) return true;
        if (!(o instanceof UniversalRadio)) return false;

        UniversalRadio that = (UniversalRadio) o;

        if (Double.compare(that.frequency, frequency) != 0) return false;
        if (display != null ? !display.equals(that.display) : that.display != null) return false;
        if (radioSwitch != null ? !radioSwitch.equals(that.radioSwitch) : that.radioSwitch != null) return false;
        if (speaker != null ? !speaker.equals(that.speaker) : that.speaker != null) return false;

        return true;
    }

    @Override
    public int hashCode() {
        int result;
        long temp;
        temp = frequency != +0.0d ? Double.doubleToLongBits(frequency) : 0L;
        result = (int) (temp ^ (temp >>> 32));
        result = 31 * result + (radioSwitch != null ? radioSwitch.hashCode() : 0);
        result = 31 * result + (speaker != null ? speaker.hashCode() : 0);
        result = 31 * result + (display != null ? display.hashCode() : 0);
        return result;
    }
}
